package com.example.hid_host_android

import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.util.Log
import android.widget.EditText
import android.widget.TextView
import androidx.appcompat.app.AppCompatActivity
import androidx.appcompat.widget.AppCompatButton

class MainActivity : AppCompatActivity() {

    private val Button by lazy {
        findViewById<AppCompatButton>(R.id.button)
    }

    private lateinit var usbHid : UsbHid

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_main)

        usbHid = UsbHid(applicationContext, 0x04d8, 0x003f).apply {
            listener = object : UsbHid.Listener {
                override fun onRunError(e: Exception) {
                    Log.w(TAG, "Occurred USB HID error.", e)
                }

                override fun onStateChanged(state: UsbHid.State) {
                    Log.d(TAG, "USB HID device state changed. state=$state")
                }

                override fun onNewData(data: ByteArray) {
                    Log.i(TAG, "Receive data from USB HID. data=${data.contentToString()}")

                    val text = findViewById<TextView>(R.id.textView)

                    // メインスレッドに処理を依頼
                    Handler(Looper.getMainLooper()).post { text.text=data.toString(Charsets.UTF_8) }

/*
                    runOnUiThread {
                        Toast
                                .makeText(this@MainActivity, data.contentToString(), Toast.LENGTH_SHORT)
                                .show()
                    }
*/
                }
            }
            openDevice()
        }

        Button.setOnClickListener { _ -> sendCommandToDevice() }
    }

    override fun onDestroy() {
        super.onDestroy()
        usbHid.closeDevice()
    }

    private fun sendCommandToDevice() {
        val editText = findViewById<EditText>(R.id.editTextSend)

     //   Toast.makeText(this, editText.text.toString(), Toast.LENGTH_SHORT).show()

        val command = ByteArray(64)

        val dummy=editText.text.toString()

        for (i in dummy.indices) {
            command[i]=dummy.toByteArray()[i]
        }

        usbHid.write(command)

    }

    companion object {
        private const val TAG = "MainActivity"
    }
}
